package util.math;
import java.util.Arrays;
import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;

public class Statistic {
		private double[] inputData; // input data
		private double medianValue; // median value
		private double meanValue; // mean of input data array
		private double maxValue; // max value of array
		private double minValue; // min value of array
		private double sdValue; // standard deviation of array
		private double sumValue; // sum of array
		
		public Statistic(double[] input) {
			this.inputData = input;
			Arrays.sort(inputData);
			double N = inputData.length;
			for(int i=0; i<inputData.length; i++) 
			{
				if(i == 0) {
					maxValue = inputData[i];
					minValue = inputData[i];
				}
				
				if(maxValue < inputData[i]) {
					maxValue = inputData[i];
				}
				
				if(minValue > inputData[i]) {
					minValue = inputData[i];
				}
				
				sumValue += inputData[i];
			}
			meanValue = sumValue/N;
			
			// if total number is odd
			// calculate standard deviation and median value
			// http://en.wikipedia.org/wiki/Standard_deviation
			if(isOdd(inputData.length)) 
			{
				medianValue = inputData[inputData.length/2];
				
			} 
			else 
			{
				double temp = inputData[inputData.length/2] + inputData[(inputData.length/2 -1)];
				medianValue = temp/2.0d;
			}
			
			double powSum = 0.0d;
			for(int k=0; k<inputData.length; k++) {
				powSum += Math.pow((inputData[k] - meanValue), 2);
			}

			// This correction (the use of N − 1 instead of N) is known as Bessel's correction
			sdValue = Math.sqrt(powSum/(N-(double)1.0d));
			
			
		}
		
		private boolean isOdd(int n) {
			if((n & 0x1) == 1)
				return true;
			else 
				return false;
		}

		public double getMedianValue() {
			return medianValue;
		}

		public double getMeanValue() {
			return meanValue;
		}

		public double getMaxValue() {
			return maxValue;
		}

		public double getMinValue() {
			return minValue;
		}

		public double getSdValue() {
			return sdValue;
		}

		public double getSumValue() {
			return sumValue;
		}
		
		public static void main(String[] args)  {
			double[] data = new double[]{15.23,12.11,7,88,17,89,6.578,13.456,9.1235,20.5678};
			// Arrays.sort(data);
			
			Statistic dsd = new Statistic(data);
			DescriptiveStatistics ds = new DescriptiveStatistics();
			for(int i=0; i<data.length; i++) 
			{
				ds.addValue(data[i]);
			}
			System.out.println("Demo sum = " + dsd.getSumValue());
			System.out.println("Demo mean = " + dsd.getMeanValue());
			System.out.println("Demo median = " + dsd.getMedianValue());
			System.out.println("Demo standard deviation = " + dsd.getSdValue());
			System.out.println("DS sum = " + ds.getSum());
			System.out.println("DS mean = " + ds.getMean());
			System.out.println("DS median = " + ds.getPercentile(50));
			System.out.println("DS standard deviation = " + ds.getStandardDeviation());
		}
		
	}
